<?php

namespace Paygenius\Paygenius\Controller\Index;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;
use Paygenius\Paygenius\Gateway\Intergration\Client;
use Paygenius\Paygenius\Gateway\Intergration\ConfirmPaymentRequest;


class Index extends Action
{
    const MERCHANT_TOKEN    = "merchant_gateway_token";

    const MERCHANT_KEY      = "merchant_gateway_key";

    const URL               = "base_api_url";

    protected $resultPageFactory;

    public function __construct(
        Context $context,
        PageFactory $resultPageFactory
    ) {
        $this->resultPageFactory = $resultPageFactory;
        parent::__construct($context);
    }

    public function execute()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();

        $customerSession = $objectManager->create('Magento\Customer\Model\Session');

        $request = $objectManager->create('\Magento\Framework\App\Request\Http');

        if ($request->getPost('MD') && $request->getPost('PaRes'))
        {
            $reference = $customerSession->getThreeDSecure();

            $customerSession->unsThreeDSecure();

            $response = $this->handleThreeDCallback($request, $reference);

            $eventManager = $objectManager->create('\Magento\Framework\Event\Manager');

            $eventManager->dispatch("threeds_assign_data", ["status"=>$response->success, "reference"=>$reference["THREEDSECURE"]["reference"]]);

            if ($response->success===true)
            {
                return $this->_redirect('checkout/onepage/success');
            }

            return $this->_redirect('checkout/onepage/failure');
        }

        return $this->resultPageFactory->create();
    }

    private function handleThreeDCallback($data, $reference)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();

        $scopeConfig = $objectManager->create('Magento\Framework\App\Config\ScopeConfigInterface');

        $key   = $scopeConfig->getValue('payment/paygenius/merchant_gateway_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $token   = $scopeConfig->getValue('payment/paygenius/merchant_gateway_token', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $logger = $objectManager->create('Psr\Log\LoggerInterface');

        $logger->info("passed the scope config situatuin".$data->getPost('PaRes'));

        $confirmPaymentRequest = new ConfirmPaymentRequest($reference["THREEDSECURE"]["reference"], $data->getPost('PaRes'));

        $apiUrl = $scopeConfig->getValue('payment/paygenius/test', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)?
                  $scopeConfig->getValue('payment/paygenius/test_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE):
                  $scopeConfig->getValue('payment/paygenius/test_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $client = new Client($token, $key, $apiUrl);

        return $client->confirmPayment($confirmPaymentRequest);
    }

}
