<?php
/*
 * Copyright (c) 2018 Methys Digital
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Methys Digital.
 * ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with
 * the terms of the license agreement you entered into with Methys Digital.
 */
namespace Paygenius\Paygenius\Gateway\Http;

use Magento\Payment\Gateway\Http\TransferBuilder;
use Magento\Payment\Gateway\Http\TransferFactoryInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Psr\Log\LoggerInterface;

class TransferFactory implements TransferFactoryInterface
{
    /**
     * @var TransferBuilder
     */
    private $transferBuilder;

    private $logger;

    /**
     * @param TransferBuilder $transferBuilder
     * @param LoggerInterface $logger
     */
    public function __construct(
        TransferBuilder $transferBuilder,
        LoggerInterface $logger
    ) {
        $this->transferBuilder = $transferBuilder;
        $this->logger = $logger;
    }

    /**
     * Builds gateway transfer object
     *
     * @param array $request
     * @return TransferInterface
     */
    public function create(array $request)
    {
        $rq = (object)$request;

        $body = $this->createPaymentbody($rq);

        $headers = $this->createHeaders($rq, $body);

        $this->logger->info("Headers aare:::", $headers);

        $this->logger->info("Dynamic URI is:::".$rq->URI);

        return $this->transferBuilder
            ->setBody($body)
            ->setMethod('POST')
            ->setHeaders($headers)
            ->setUri($rq->URI)
            ->build();
    }

    private function createPaymentbody($request)
    {
        switch ($request->TXN_TYPE)
        {
            case "A":

                $creditCard = $this->constructCreditCard($request);

                $transaction = $this->constructTransaction($request);

                $consumer    = $this->constructConsumer($request);

                $paymentRequest = [
                    'creditCard'=>$creditCard,
                    'transaction'=>$transaction,
                    'consumer'=>$consumer,
                    "threeDSecure"=>false,
                    "token"=>$request->MERCHANT_TOKEN,
                    "key"=>$request->MERCHANT_KEY
                ];

                return $paymentRequest;

                break;

            default:

                return ['damn.'

                ];
                break;
        }
    }

    private function createHeaders($request, $body)
    {

        return array(
            "content-type: application/json",
            "x-token: ".$request->MERCHANT_TOKEN,
        );

    }

    private function constructTransaction($request)
    {
        return [
            'reference'=>$request->INVOICE,
            'description'=>'Purchase via Paygenius magento.',
            'currency'=>$request->CURRENCY,
            'amount'=>(double)$request->AMOUNT*100
        ];
    }

    private function constructConsumer($request)
    {
            return [
                'name'=>$request->CONSUMER_NAME,
                'surname'=>$request->CONSUMER_SURNAME,
                'email'=>$request->EMAIL
                ];
    }

    private function constructCreditCard($request)
    {
        return [
            'number'=>$request->CC_NUMBER,
            'cardHolder'=>"New",
            'expiryYear'=>$request->CC_EXP_YEAR,
            'expiryMonth'=>$request->CC_EXP_MONTH,
            'cvv'=>$request->CVV
        ];
    }

}
