<?php
/*
 * Copyright (c) 2018 Methys Digital
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Methys Digital.
 * ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with
 * the terms of the license agreement you entered into with Methys Digital.
 */
namespace Paygenius\Paygenius\Gateway\Request;

use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class AuthorizationRequest implements BuilderInterface
{
    private $logger;

    private $scopeConfig;

    /**
     * @param LoggerInterface $logger
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        LoggerInterface $logger,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Builds ENV request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject)
    {
        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();
        $address = $order->getBillingAddress();
        $pay     = $payment->getPayment();
        $merchant_key   = $this->scopeConfig->getValue('payment/paygenius/merchant_gateway_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $merchant_token   = $this->scopeConfig->getValue('payment/paygenius/merchant_gateway_token', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $isTest   = $this->scopeConfig->getValue('payment/paygenius/test', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $currency   = $this->scopeConfig->getValue('payment/paygenius/trading_currency', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $uri    = $this->resolveEnvironment($isTest);
        $result =  [
            'TXN_TYPE' => 'A',
            'INVOICE' => $order->getOrderIncrementId(),
            'AMOUNT' => $order->getGrandTotalAmount(),
            'CURRENCY' => $currency,
            'EMAIL' => $address->getEmail(),
            'CONSUMER_NAME'=>$address->getFirstname(),
            'CONSUMER_SURNAME'=>$address->getLastname(),
            'CC_NUMBER'=>$pay->getAdditionalInformation('creditCardNumber'),
            'CC_EXP_YEAR'=>$pay->getAdditionalInformation('creditCardExpYear'),
            'CC_EXP_MONTH'=>$pay->getAdditionalInformation('creditCardExpMonth'),
            'CVV'=>$pay->getAdditionalInformation('creditCardVerificationNumber'),
            'MERCHANT_KEY'=>$merchant_key,
            'MERCHANT_TOKEN'=>$merchant_token,
            'URI'=>$uri
        ];

        $this->logger->info('Authorization Request:', $result);

        return $result;
    }

    private function resolveEnvironment($istest)
    {
        return $istest?$this->scopeConfig->getValue('payment/paygenius/test_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE):$this->scopeConfig->getValue('payment/paygenius/live_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

}
