<?php
/*
 * Copyright (c) 2018 Methys Digital
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Methys Digital.
 * ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with
 * the terms of the license agreement you entered into with Methys Digital.
 */
namespace Paygenius\Paygenius\Observer;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Webapi\Exception;
use Paygenius\Paygenius\Gateway\Intergration\Client;
use Paygenius\Paygenius\Gateway\Intergration\ExecutePaymentRequest;


class PaygeniusThreedsAssign implements ObserverInterface {

    const PAYMENT_FAILED_STATE = "payment failed";

    protected $logger;

    private $url;

    private $responseFactory;

    public function __construct(
        \Psr\Log\LoggerInterface $loggerInterface,
        \Magento\Framework\App\ResponseFactory $responseFactory,
        \Magento\Framework\UrlInterface $url
    )
    {
        $this->logger = $loggerInterface;
        $this->url = $url;
        $this->responseFactory = $responseFactory;
    }


    public function execute( Observer $observer ) {

        $objectManager = ObjectManager::getInstance();

        $checkoutSession = $objectManager->create("\Magento\Checkout\Model\Session");

        $order = $checkoutSession->getLastRealOrder();

        if ($observer->getData("status") === false)
        {
            $scopeConfig = $objectManager->create('Magento\Framework\App\Config\ScopeConfigInterface');

            $key   = $scopeConfig->getValue('payment/paygenius/merchant_gateway_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

            $token   = $scopeConfig->getValue('payment/paygenius/merchant_gateway_token', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

            $api_url = $scopeConfig->getValue('payment/paygenius/test', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)?
                $scopeConfig->getValue('payment/paygenius/test_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE):
                $scopeConfig->getValue('payment/paygenius/live_api_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

            $executePaymentRequest = new ExecutePaymentRequest(trim($observer->getData("reference")));

            $client = new Client($token, $key, $api_url);

            $this->logger->info("the capture url is:::".$api_url);

            $this->logger->info("the capture Reference is:::".$observer->getData("reference"));

            try
            {
                $captureResponse = $client->executePayment($executePaymentRequest);
            }
            catch (Exception $e)
            {
                $this->logger->info("Problem encountered by server capturing funds.");

                $this->handleFailure($order);

                exit;
            }

            $this->logger->info("Capture response is::::::.".json_encode($captureResponse));

            $captureResponse->success ? $this->handleSuccess($order):$this->handleFailure($order);
        }

        else
        {
            $this->handleFailure($order);
        }


    }

    private function handleSuccess($order)
    {
        $order->setState(\Magento\Sales\Model\Order::STATE_COMPLETE, true);

        $order->setStatus(\Magento\Sales\Model\Order::STATE_COMPLETE);

        $order->addStatusToHistory($order->getStatus(), 'Payment for order was successful.');

        $order->save();

        return true;
    }

    private function handleFailure($order)
    {
        $order->setState(\Magento\Sales\Model\Order::STATE_CANCELED, true);

        $order->setStatus(\Magento\Sales\Model\Order::STATE_CANCELED);

        $order->addStatusToHistory($order->getStatus(), 'Payment for order was unsuccessful. 3ds failure.');

        $order->save();

        return true;
    }
}