define(
    [
        'underscore',
        'Magento_Checkout/js/view/payment/default',
        'Magento_Payment/js/model/credit-card-validation/credit-card-data',
        'Magento_Payment/js/model/credit-card-validation/credit-card-number-validator',
        'mage/translate',
        'Magento_Checkout/js/checkout-data',
        'Magento_Checkout/js/model/quote',
        'mage/url',
        'Magento_Payment/js/model/credit-card-validation/validator',
        'mage/validation',
        'jquery',
        'ko'
    ],
    function (_, Component, creditCardData, url, cardNumberValidator, $t, $, ko, urlBuilder) {
        'use strict';

        return Component.extend({
            redirectAfterPlaceOrder: false,
            defaults: {
                template: 'Paygenius_Paygenius/payment/form',
                creditCardType: '',
                creditCardExpYear: '',
                creditCardExpMonth: '',
                creditCardNumber: '',
                creditCardSsStartMonth: '',
                creditCardSsStartYear: '',
                creditCardVerificationNumber: '',
                selectedCardType: null,
                THREEDSECURE:''
            },
            initObservable: function () {
                this._super()
                    .observe([
                        'transactionResult',
                        'creditCardType',
                        'creditCardExpYear',
                        'creditCardExpMonth',
                        'creditCardNumber',
                        'creditCardVerificationNumber',
                        'creditCardSsStartMonth',
                        'creditCardSsStartYear',
                        'selectedCardType',
                        'THREEDSECURE'
                    ]);
                return this;
            },

            afterPlaceOrder: function() {

                var getUrl = window.location;

                var baseUrl = getUrl .protocol + "//" + getUrl.host;

                window.location.replace(baseUrl+'/paygeniusthreed/');
            },


            getCode: function() {
                return 'paygenius';
            },

            getData: function() {
                return {
                    'method': this.item.method,
                    'additional_data': {
                        'creditCardType': this.creditCardType(),
                        'creditCardExpYear':this.creditCardExpYear(),
                        'creditCardExpMonth':this.creditCardExpMonth(),
                        'creditCardVerificationNumber':this.creditCardVerificationNumber(),
                        'creditCardNumber':this.creditCardNumber(),
                        'THREEDSECURE':this.THREEDSECURE()
                    }
                };
            },

            initialize: function() {
                var self = this;
                this._super();

                //Set credit card number to credit card data object
                this.creditCardNumber.subscribe(function(value) {
                    var result;
                    self.selectedCardType(null);

                    if (value == '' || value == null) {
                        return false;
                    }
                    result = cardNumberValidator(value);

                    if (!result.isPotentiallyValid && !result.isValid) {
                        return false;
                    }
                    if (result.card !== null) {
                        self.selectedCardType(result.card.type);
                        creditCardData.creditCard = result.card;
                    }

                    if (result.isValid) {
                        creditCardData.creditCardNumber = value;
                        self.creditCardType(result.card.type);
                    }
                });

                //Set expiration year to credit card data object
                this.creditCardExpYear.subscribe(function(value) {
                    creditCardData.expirationYear = value;
                });

                //Set expiration month to credit card data object
                this.creditCardExpMonth.subscribe(function(value) {
                    creditCardData.expirationYear = value;
                });

                //Set cvv code to credit card data object
                this.creditCardVerificationNumber.subscribe(function(value) {
                    creditCardData.cvvCode = value;
                });
            },

            isActive: function () {
                return true;
            },

            getCcAvailableTypes: function() {
                return window.checkoutConfig.payment['paygenius'].availableTypes;
            },

            getPaymentAcceptanceMarkSrc: function () {
                return window.checkoutConfig.payment['paygenius'].paymentAcceptanceMarkSrc;
            },

            getCcMonths: function() {
                return window.checkoutConfig.payment['paygenius'].months;
            },

            getCcYears: function() {
                return window.checkoutConfig.payment['paygenius'].years;
            },

            hasVerification: function() {
                return window.checkoutConfig.payment['paygenius'].hasVerification;
            },

            getCcAvailableTypesValues: function() {
                return _.map(this.getCcAvailableTypes(), function(value, key) {
                    return {
                        'value': key,
                        'type': value
                    }
                });
            },
            getCcMonthsValues: function() {
                return _.map(this.getCcMonths(), function(value, key) {
                    return {
                        'value': key,
                        'month': value
                    }
                });
            },
            getCcYearsValues: function() {
                return _.map(this.getCcYears(), function(value, key) {
                    return {
                        'value': key,
                        'year': value
                    }
                });
            },

            getTransactionResults: function() {
                return _.map(window.checkoutConfig.payment['paygenius'].transactionResults, function(value, key) {
                    return {
                        'value': key,
                        'transaction_result': value
                    }
                });
            }
        });
    }
);